/*
 *  Copyright (c) 2010-2013, Rutgers, The State University of New Jersey
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *      * Neither the name of the organization(s) stated above nor the
 *        names of its contributors may be used to endorse or promote products
 *        derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file   mfaggregator.h
 * @author wontoniii (bronzino@winlab.rutgers.edu)
 * @date   July, 2011
 * @brief  Temp.
 *
 * Temp
 */

#ifndef MF_AGGREGATOR_H_
#define MF_AGGREGATOR_H_

#include <map>

#include "mftypes.h"
#include "mfchunkinfo.h"
#include "mfdeviceinfo.h"
#include "mfbuffer.h"
#include "mfeventqueue.h"

using namespace std;

class MF_Aggregator {
	//Using a buffer to store data for the incoming chunks
	MF_Buffer mPool;
	//Outstanding chunks on the interface. The key is the hopID
	map <u_int, MF_ChunkInfo *> outsChunks;
	u_int lastHopID;
	MF_EventQueue *eq;
	
	void tryRelease(u_int);
	void tryRelease(MF_ChunkInfo *);
	
public:
	MF_Aggregator();
	MF_Aggregator(MF_EventQueue *eq);
	MF_Aggregator(MF_EventQueue *eq, int buffSize);
	~MF_Aggregator();
	void handleData(const u_char *);
	int handleCsyn(u_int, u_int);
	void sendAck();
	void sendCtlMsg(u_char *, u_int);
	MF_ChunkInfo *getChunkInfo(u_int);
	void releaseChunk(u_int);
};

#endif /* MF_AGGREGATOR_H_ */