/*
 *  Copyright (c) 2010-2013, Rutgers, The State University of New Jersey
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *      * Neither the name of the organization(s) stated above nor the
 *        names of its contributors may be used to endorse or promote products
 *        derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file   mfsocketmanager.h
 * @author wontoniii (bronzino@winlab.rutgers.edu)
 * @date   August, 2013
 * @brief  Class that handles the tresources for each socket.
 *
 * Class that handles the tresources for each socket.
 */

#ifndef MF_SOCKET_MAN
#define MF_SOCKET_MAN

#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <list>
#include <vector>
#include <queue>
#include <map>

//TODO: remove once found clean solution to pass info about GUID
#include <include/mfroutingheader.hh>
#include <include/mfguid.hh>

#include "mferrors.h"
#include "mftypes.h"
#include "mfbuffer.h"
#include "mfchunkinfo.h"
#include "mfstackevent.h"
#include "mfserver.h"
#include "mftransport.h"
#include "mfsockettable.h"

using namespace std;

class MF_SocketTable;

class MF_SocketManager{
	//Resources to communicate with api
	struct sockaddr_un cliAddr;
	int mDataSock, mConnSock;
	u_int ID;
	u_int lastGetRequest;
	
	//Socket resources
	//TODO at the moment it distinguish only between content socket and data socket, this will need a new design
	int type; //1 data 2 content
	MF_Buffer buffer;
	list <int> GUIDs;
	list <u_int> TIDs;
	vector<MF_APIRecv *> pendingRec;
	queue<MF_APISend *> pendingSend;
	//Content request data
	map<unsigned int, unsigned int> pendingGet;
	//incoming get requests
	queue<vector<u_char *> *> pendingGetRequest;
	queue<u_int> pendingGetRequestSizes;
	queue<MF_APIDoGet *> pendingDoGet;
	map<unsigned int, unsigned int> pendingDoGetReply;
	queue<MF_APIGetResponse *> pendingGetResponse;
	
	//Other classes involved
	MF_Server *ms;
	MF_EventQueue *eq;
	MF_SocketTable *st;
	MF_Transport *trnsp; //transport. At the moment it will only be a single transport (basic). In the future the transport/s will be selected based on the policy
	bool open;

	void sendReply(void *, u_int, u_int);
	void sendReplyRec(void *buffer, u_int size, u_int pid);
	int recvSinglePacket(u_char *buf, int size, int offset);
	int recvData(vector<u_char *> *chkPkt, u_int chunkSize);
	void sendData(vector<u_char *> *chkPkt, u_int size);
	void newSpaceAvail();
	void sendGetMessage(vector<u_char *> *v, u_int UID, u_int reqID, u_int size, u_int dstGUID, mfflag_t opts);
	void sendGetResponseMessage(vector<u_char *> *v, u_int UID, u_int reqID, u_int size, u_int srcGUID, u_int dstGUID, mfflag_t opts);
	void sendMessage(vector<u_char *> *v, u_int UID, u_int reqID, u_int size, u_int dstGUID, mfflag_t opts);
	void replyDoGetRequest(MF_APIDoGet *event);
	
	//TODO this will need to be removed
	void basicMsgAvailable(MF_EvUpTransp *event);
	void contentMsgAvailable(MF_EvUpTransp *event);
	
public:
	MF_SocketManager();
	MF_SocketManager(MF_SocketTable *st, MF_EventQueue *eq, MF_Server *ms, u_int UID);
	MF_SocketManager(MF_SocketTable *st, MF_EventQueue *eq, MF_Server *ms, u_int UID, int nChunks);
	~MF_SocketManager();
	
	bool init();
	int connectToClient();
	bool isOpen();
	
	u_int getID();
	void setID(u_int);
	void addGUID(int);
	void removeGUID(int);
	void addTID(u_int);
	void removeTID(u_int);
	
	
	list<int>::iterator getGUIDBegin();
	list<int>::iterator getGUIDEnd();
	list<u_int>::iterator getTIDBegin();
	list<u_int>::iterator getTIDEnd();
	
	void msgAvailable(MF_EvUpTransp *);
	
	void openReplySuccess();
	void openReplyError();
	int openRequest(const char *profile, mfflag_t opts, const int GUID);
	void sendRequest(MF_APISend *);
	void recvRequest(MF_APIRecv *);
	void getRequest(MF_APIGet *);
	void doGetRequest(MF_APIDoGet *);
	void getResponseRequest(MF_APIGetResponse *);
	void attachComplete();
	void detachComplete();
	void closeRequest();
	
	MF_Transport *getTransportByTID(u_int);
	void releaseChunk(MF_ChunkInfo *ci);
};

#endif /* MF_SOCKET_MAN */
